# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator/create_python_api.py script.
"""Operations for manipulating the binary representations of integers.
"""

import sys as _sys

from tensorflow.python.ops.gen_bitwise_ops import bitwise_and
from tensorflow.python.ops.gen_bitwise_ops import bitwise_or
from tensorflow.python.ops.gen_bitwise_ops import bitwise_xor
from tensorflow.python.ops.gen_bitwise_ops import invert
from tensorflow.python.ops.gen_bitwise_ops import left_shift
from tensorflow.python.ops.gen_bitwise_ops import right_shift